
package com.redxun.knowledge.map.service;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.entity.IUser;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.IdGenerator;
import com.redxun.common.utils.ContextUtil;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.vo.DicVo;
import com.redxun.knowledge.entity.vo.KnowledgeCategoryAdminVo;
import com.redxun.knowledge.map.entity.dao.*;
import com.redxun.knowledge.map.entity.dao.Map;
import com.redxun.knowledge.map.entity.dto.MapContentDto;
import com.redxun.knowledge.map.entity.dto.MapDto;
import com.redxun.knowledge.map.entity.dto.MapRouteDto;
import com.redxun.knowledge.map.entity.vo.*;
import com.redxun.knowledge.map.mapper.*;
import com.redxun.knowledge.service.CommonServiceImpl;
import com.redxun.knowledge.service.KnowledgeCategoryServiceImpl;
import com.redxun.knowledge.service.KnowledgeServiceImpl;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * [业务--知识地图表]业务服务类
 */
@Service
public class MapServiceImpl extends SuperServiceImpl<MapMapper, Map> implements BaseService<Map> {

    @Resource
    private MapMapper mapMapper;

    @Autowired
    private MapRouteMapper mapRouteMapper;

    @Autowired
    private MapContentMapper mapContentMapper;

    @Autowired
    private MapGroupPurviewUserMapper mapGroupPurviewUserMapper;

    @Autowired
    private MapLearningMapper mapLearningMapper;

    @Autowired
    private UserService userService;

    @Autowired
    private CommonServiceImpl commonService;

    @Autowired
    private KnowledgeServiceImpl knowledgeService;

    @Autowired
    private ThreadPoolExecutor threadPoolExecutor;

    @Autowired
    private KnowledgeCategoryServiceImpl knowledgeCategoryService;

    @Override
    public BaseDao<Map> getRepository() {
        return mapMapper;
    }

    /**
     * 添加地图或修改地图
     *
     * @param mapDto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String create(MapDto mapDto) {
        // 向地图表中添加数据
        Map map = new Map();
        BeanUtils.copyProperties(mapDto, map);
        if (mapDto.getPkId() == null) {
            map.setPkId(IdGenerator.getIdStr());
            map.setUpdateBy(userService.queryLoginUser());
            mapMapper.insert(map);
        } else {
            mapMapper.updateById(map);
        }
        Map mapById = mapMapper.selectById(map.getPkId());
        //向可阅读者表中添加数据
        List<String> purviewUsers = mapDto.getPurviewUsers();
        if (!CollectionUtils.isEmpty(purviewUsers)){
            mapGroupPurviewUserMapper.deleteBatchIds(Arrays.asList(mapById.getPkId()));
            purviewUsers.forEach(purviewUser -> {
                MapGroupPurviewUser mapGroupPurviewUser = new MapGroupPurviewUser();
                mapGroupPurviewUser.setPkId(IdGenerator.getIdStr());
                mapGroupPurviewUser.setMapId(mapById.getPkId());
                mapGroupPurviewUser.setUserId(purviewUser);
                mapGroupPurviewUser.setUpdateBy(userService.queryLoginUser());
                mapGroupPurviewUserMapper.insert(mapGroupPurviewUser);
            });
        } else {
            mapGroupPurviewUserMapper.deleteBatchIds(Arrays.asList(mapById.getPkId()));
        }

        // 向学习路径表中添加数据
        if (!CollectionUtils.isEmpty(mapDto.getRoutes())) {
            mapRouteMapper.deleteBatchIds(Arrays.asList(mapById.getPkId()));
            mapContentMapper.deleteBatchIds(Arrays.asList(mapById.getPkId()));
            mapLearningMapper.deleteBatchIds(Arrays.asList(mapById.getPkId()));
            List<MapRouteDto> routes = mapDto.getRoutes();
            //排序处理
            AtomicInteger sortRoute = new AtomicInteger(1);
            routes.forEach(route -> {
                MapRoute mapRoute = new MapRoute();
                BeanUtils.copyProperties(route, mapRoute);
                mapRoute.setPkId(IdGenerator.getIdStr());
                mapRoute.setMapId(mapById.getPkId());
                mapRoute.setUpdateBy(userService.queryLoginUser());
                mapRoute.setSort(sortRoute.getAndIncrement());
                mapRouteMapper.insert(mapRoute);
                MapRoute mapRouteById = mapRouteMapper.selectById(mapRoute.getPkId());
                // 向学习内容表中添加数据
                if (!CollectionUtils.isEmpty(route.getContents())) {
                    List<MapContentDto> contents = route.getContents();
                    //排序处理
                    AtomicInteger sortMapContent = new AtomicInteger(1);
                    contents.forEach(content -> {
                        MapContent mapContent = new MapContent();
                        BeanUtils.copyProperties(content, mapContent);
                        mapContent.setPkId(IdGenerator.getIdStr());
                        mapContent.setMapId(mapRouteById.getMapId());
                        mapContent.setRouteId(mapRouteById.getPkId());
                        mapContent.setSort(sortMapContent.getAndIncrement());
                        mapContent.setUpdateBy(userService.queryLoginUser());
                        mapContentMapper.insert(mapContent);
                    });
                } else {
                    //清除路径下的信息
                    mapContentMapper.deleteByRouteId(mapRoute.getPkId());
                    mapLearningMapper.deleteByRouteId(mapRoute.getPkId());
                }
            });
        } else {
            mapRouteMapper.deleteBatchIds(Arrays.asList(mapById.getPkId()));
            mapContentMapper.deleteBatchIds(Arrays.asList(mapById.getPkId()));
            mapLearningMapper.deleteBatchIds(Arrays.asList(mapById.getPkId()));
        }
        return mapById.getPkId();
    }

    /**
     * 分页查询地图列表
     *
     * @param queryFilter
     * @return
     */
    public IPage query(QueryFilter queryFilter) {
        java.util.Map<String, Object> params = PageHelper.constructParams(queryFilter);
        IPage page = mapMapper.findAllMap(queryFilter.getPage(), queryFilter.getParams(), params);
        List<DicVo> dicVoList = commonService.queryDic("KNOWLEDGE_MAP");
        List<MapVo> records = page.getRecords();
        records.forEach(e -> {
            String typeName = e.getTypeName();
            dicVoList.forEach(dicVo -> {
                if (dicVo.getValue().equals(typeName)) {
                    e.setTypeName(dicVo.getName());
                }
            });
        });
        return page;
    }

    /**
     * 管理端查看地图详情
     *
     * @param pkId
     * @return
     */
    public MapVo data(String pkId) {
        MapVo mapVo = new MapVo();
        Map map = mapMapper.selectById(pkId);
        BeanUtils.copyProperties(map, mapVo);
        List<DicVo> dicVoList = commonService.queryDic("KNOWLEDGE_MAP");
        dicVoList.forEach(dicVo -> {
            if (map.getType().toString().equals(dicVo.getValue())) {
                mapVo.setTypeName(dicVo.getName());
            }
        });
        List<MapRoute> mapRouteList = mapRouteMapper.selectList(new LambdaQueryWrapper<MapRoute>().eq(MapRoute::getMapId, mapVo.getPkId()).orderByAsc(MapRoute::getSort));
        if (!CollectionUtils.isEmpty(mapRouteList)) {
            List<MapRouteVo> mapRouteVoList = mapRouteList.stream().map(mapRoute -> {
                MapRouteVo mapRouteVo = new MapRouteVo();
                BeanUtils.copyProperties(mapRoute, mapRouteVo);
                return mapRouteVo;
            }).collect(Collectors.toList());
            mapVo.setRoutes(mapRouteVoList);
            mapRouteVoList.forEach(mapRouteVo -> {
                List<MapContent> mapContentList = mapContentMapper.findAllMapContent(mapRouteVo.getPkId());
                //过滤掉知识ID不为空且知识被删除的数据
                List<MapContent> collect = mapContentList.stream().
                        filter(e -> !(e.getKnowledgeId() != null && e.getKnowledgeIsDel() == null)).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(collect)) {
                    List<MapContentVo> mapContentVoList = collect.stream().map(mapContent -> {
                        MapContentVo mapContentVo = new MapContentVo();
                        BeanUtils.copyProperties(mapContent, mapContentVo);
                        if (mapContent.getKnowledgeId() != null) {
                            Knowledge knowledge = knowledgeService.get(mapContent.getKnowledgeId());
                            if (knowledge != null) {
                                String photo = (String) userService.querySexAndPhoto(knowledge.getCreateBy()).get("photo");
                                mapContentVo.setAuthorHead(photo);
                                mapContentVo.setTitles(knowledge.getTitles());
                                mapContentVo.setSummary(knowledge.getSummary());
                                mapContentVo.setAuthor(knowledge.getAuthor());
                                mapContentVo.setCreateTime(knowledge.getCreateTime());
                                mapContentVo.setViews(knowledge.getViews());
                                mapContentVo.setAttachment(knowledge.getAttachment());
                                // 有文件,截取文件名后缀
                                String attachmentName = knowledge.getAttachmentName();
                                if (attachmentName != null){
                                    String[] strArray = attachmentName.split("\\.");
                                    int suffixIndex = strArray.length - 1;
                                    //如果是word文件,后缀名修改为word
                                    if(strArray[suffixIndex].equals("doc") || strArray[suffixIndex].equals("docx")){
                                        strArray[suffixIndex] = "word";
                                    }
                                    mapContentVo.setAttachmentType(strArray[suffixIndex]);
                                }
                                //添加分类信息
                                KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryService.findAllParentByCategoryId(knowledge.getCategoryId());
                                String name3 = knowledgeCategoryAdminVo.getName();
                                mapContentVo.setCategory(name3);
                                if (knowledgeCategoryAdminVo.getParents() != null){
                                    String name2 = knowledgeCategoryAdminVo.getParents().getName();
                                    mapContentVo.setCategory(name2 + " " + name3);
                                    if (knowledgeCategoryAdminVo.getParents().getParents() != null){
                                        String name1 = knowledgeCategoryAdminVo.getParents().getParents().getName();
                                        mapContentVo.setCategory(name1 + " " +name2 + " " + name3);
                                    }
                                }
                            }
                        }
                        return mapContentVo;
                    }).collect(Collectors.toList());
                    mapRouteVo.setContents(mapContentVoList);
                }
            });
        }
        return mapVo;
    }

    /**
     * 管理端删除地图
     *
     * @param entities
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Collection<Serializable> entities) {
        //删除学习内容表中数据
        mapContentMapper.deleteBatchIds(entities);
        //删除学习路径表中数据
        mapRouteMapper.deleteBatchIds(entities);
        //删除可查看员工表中数据
        mapGroupPurviewUserMapper.deleteBatchIds(entities);
        //删除学习记录表中数据
        mapLearningMapper.deleteBatchIds(entities);
        //删除地图表中数据
        this.getRepository().deleteBatchIds(entities);
    }

    /**
     * PC端根据地图类型字典值获取知识地图列表数据
     *
     * @param type
     * @return
     */
    public List<java.util.Map<String, Object>> listOfType(Integer type) {
        List<Map> mapList = mapMapper.selectList(new LambdaQueryWrapper<Map>().eq(Map::getType, type).orderByDesc(Map::getCreateTime));
        List<java.util.Map<String, Object>> listResult = new ArrayList<>();
        mapList.forEach(map -> {
            java.util.Map<String, Object> mapResult = new HashMap<>();
            if (hasRole(map.getPkId())) {
                mapResult.put("pkId", map.getPkId());
                mapResult.put("name", map.getName());
                listResult.add(mapResult);
            }
        });
        return listResult;
    }

    /**
     * PC端根据知识地图ID获取知识地图配置的学习路径列表数据(加入权限判断)
     *
     * @param mapId
     * @return
     */
    public List<HashMap<String, Object>> routes(String mapId) {
        if (hasRole(mapId)) {
            List<MapRoute> mapRouteList = mapRouteMapper.selectList(new LambdaQueryWrapper<MapRoute>().eq(MapRoute::getMapId, mapId).orderByAsc(MapRoute::getSort));
            List<HashMap<String, Object>> result = mapRouteList.stream().map(mapRoute -> {
                HashMap<String, Object> hashMap = new HashMap<>();
                hashMap.put("pkId", mapRoute.getPkId());
                hashMap.put("name", mapRoute.getName());
                return hashMap;
            }).collect(Collectors.toList());
            return result;
        }
        return null;
    }

    /**
     * 设置地图学习内容学习状态-PC端
     *
     * @param pkId
     */
    public void updateLearningState(String pkId) {
        String mapId = mapContentMapper.selectById(pkId).getMapId();
        if (hasRole(mapId)) {
            if (mapContentMapper.selectById(pkId) != null) {
                MapLearning mapLearning = new MapLearning();
                mapLearning.setPkId(IdGenerator.getIdStr());
                mapLearning.setMapId(mapId);
                mapLearning.setRouteId(mapContentMapper.selectById(pkId).getRouteId());
                mapLearning.setContentId(pkId);
                mapLearning.setUserId(userService.queryLoginUser());
                mapLearningMapper.insert(mapLearning);
            }
        }
    }

    /**
     * PC端根据知识地图ID获取知识地图详情数据
     *
     * @param pkId
     * @return
     */
    public MapPcVo info(String pkId) {
        if (hasRole(pkId)) {
            String userId = userService.queryLoginUser();
            CompletableFuture<MapPcVo> mapPcVoCompletableFuture = CompletableFuture.supplyAsync(() -> {
                MapPcVo mapPcVo = mapMapper.info(pkId);
                return mapPcVo;
            }, threadPoolExecutor);
            CompletableFuture<Void> titleCompletableFuture = mapPcVoCompletableFuture.thenAcceptAsync(mapPcVo -> {
                List<DicVo> dicVoList = commonService.queryDic("KNOWLEDGE_MAP");
                // 地图名称显示处理
                Integer type = mapPcVo.getType();
                dicVoList.forEach(dicVo -> {
                    if (dicVo.getValue().equals(type.toString())) {
                        mapPcVo.setTitle(dicVo.getName() + "-" + mapPcVo.getName());
                    }
                });
            }, threadPoolExecutor);
            //查看相关知识
            CompletableFuture<Void> knowledgeCompletableFuture = mapPcVoCompletableFuture.thenAcceptAsync(mapPcVo -> {
                AtomicReference<Integer> count = new AtomicReference<>(0);
                List<MapRouteVo> routes = mapPcVo.getRoutes();
                routes.forEach(route -> {
                    //设置学习路径状态
                    route.setState(0);
                    List<MapContent> contentList = mapContentMapper.findAllMapContent(route.getPkId());
                    //过滤掉知识ID不为空且知识被删除的数据
                    List<MapContent> collect = contentList.stream().
                            filter(e -> !(e.getKnowledgeId() != null && e.getKnowledgeIsDel() == null)).collect(Collectors.toList());
                    if (!CollectionUtils.isEmpty(collect)){
                        List<MapContentVo> contents = collect.parallelStream().map(mapContent -> {
                            MapContentVo content = new MapContentVo();
                            BeanUtils.copyProperties(mapContent, content);
                            //查看知识
                            if (content.getKnowledgeId() != null) {
                                Knowledge knowledge = knowledgeService.get(content.getKnowledgeId());
                                if (knowledge != null) {
                                    String photo = (String) userService.querySexAndPhoto(knowledge.getCreateBy()).get("photo");
                                    content.setAuthorHead(photo);
                                    content.setTitles(knowledge.getTitles());
                                    content.setSummary(knowledge.getSummary());
                                    content.setAuthor(knowledge.getAuthor());
                                    content.setCreateTime(knowledge.getCreateTime());
                                    content.setViews(knowledge.getViews());
                                    content.setAttachment(knowledge.getAttachment());
                                    // 有文件,截取文件名后缀
                                    String attachmentName = knowledge.getAttachmentName();
                                    if (attachmentName != null){
                                        String[] strArray = attachmentName.split("\\.");
                                        int suffixIndex = strArray.length - 1;
                                        content.setAttachmentType(strArray[suffixIndex]);
                                    }
                                    //添加分类信息
                                    KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryService.findAllParentByCategoryId(knowledge.getCategoryId());
                                    String name3 = knowledgeCategoryAdminVo.getName();
                                    content.setCategory(name3);
                                    if (knowledgeCategoryAdminVo.getParents() != null){
                                        String name2 = knowledgeCategoryAdminVo.getParents().getName();
                                        content.setCategory(name2 + " " + name3);
                                        if (knowledgeCategoryAdminVo.getParents().getParents() != null){
                                            String name1 = knowledgeCategoryAdminVo.getParents().getParents().getName();
                                            content.setCategory(name1 + " " +name2 + " " + name3);
                                        }
                                    }
                                }
                            }
                            //设置学习内容状态
                            if (updateState(userId, pkId, route.getPkId(), content.getPkId()) > 0) {
                                content.setState(1);
                            } else {
                                content.setState(0);
                            }
                            return content;
                        }).collect(Collectors.toList());

                        if (contents.stream().allMatch(e -> e.getState() == 1)) {
                            route.setState(1);
                        }
                        route.setContents(contents);
                        //知识收录量
                        count.set(count.get() + contents.size());
                    } else {
                        route.setContents(new ArrayList<>());
                    }
                });
                mapPcVo.setContentTotal(count.get());
            }, threadPoolExecutor);
            CompletableFuture.allOf(mapPcVoCompletableFuture,
                    titleCompletableFuture,
                    knowledgeCompletableFuture).join();
            return mapPcVoCompletableFuture.join();
        }
        return null;
    }

    //变更学习状态(逻辑变更)
    public Integer updateState(String userId, String mapId, String routeId, String contentId) {
        QueryWrapper<MapLearning> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("USER_ID", userId);
        queryWrapper.eq("MAP_ID", mapId);
        queryWrapper.eq("ROUTE_ID", routeId);
        queryWrapper.eq("CONTENT_ID", contentId);
        Integer count = mapLearningMapper.selectCount(queryWrapper);
        return count;
    }


    /**
     * 权限判断方法(知识地图PC端使用)
     * @param mapId
     * @return
     */
    public boolean hasRole(String mapId) {
        //权限判断
        Map map = mapMapper.selectById(mapId);
        //当前登录用户的部门Id
        String deptId = userService.queryLoginUserDeptId();
        //地图对应的部门权限
        Integer count = 0;
        List<String> groupIds = Arrays.stream(map.getGroupId().split(",")).collect(Collectors.toList());
        for (String groupId : groupIds) {
            count = mapMapper.selectAllByDeptId(deptId, groupId);
            if (count > 0) {
                break;
            }
        }
        //包含登录人的部门
        if (count > 0) {
            if (map.getGroupPurview() == 1) {
                return true;
            } else {
                QueryWrapper<MapGroupPurviewUser> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("MAP_ID", map.getPkId());
                queryWrapper.eq("USER_ID", userService.queryLoginUser());
                // 查询结果>0,说明该员工可见
                Integer countPurview = mapGroupPurviewUserMapper.selectCount(queryWrapper);
                return countPurview > 0;
            }
        }
        return false;
    }

    /**
     * 根据搜索条件（地图名称、所属组织、地图类型、操作人和操作时间）分页查询地图列表数据
     * @param filter
     * @param params
     * @return
     */
    public IPage<OsUserVo> queryGroupUsers(QueryFilter filter, java.util.Map params) {
        String groupIds = (String)params.get("groupIds");
        if (StringUtils.isNotEmpty(groupIds)) {
            if ("curOrg".equals(groupIds)) {
                IUser user = ContextUtil.getCurrentUser();
                if (user != null) {
                    groupIds = user.getDeptId();
                }
            }

            String[] groupIdList = groupIds.split(",");
            params.put("groupIdList", Arrays.asList(groupIdList));
        }

        IPage<OsUserVo> osUserIPage = mapMapper.queryGroupUsers(filter.getPage(), params);
        List<OsUserVo> osUsers = osUserIPage.getRecords();
        osUsers.forEach(e -> e.setGroupSummary(mapGroupPurviewUserMapper.findGroupSummary(e.getUserId())));
        return osUserIPage;
    }

    /**
     * 根据知识ID获取知识快照信息
     * @param pkId
     * @return
     */
    public KnowledgeVo knowledgeSnapshot(String pkId) {
        KnowledgeVo knowledgeVo = new KnowledgeVo();
        Knowledge knowledge = knowledgeService.get(pkId);
        String photo = (String) userService.querySexAndPhoto(knowledge.getCreateBy()).get("photo");
        knowledgeVo.setPkId(pkId);
        knowledgeVo.setAuthorHead(photo);
        knowledgeVo.setTitles(knowledge.getTitles());
        knowledgeVo.setSummary(knowledge.getSummary());
        knowledgeVo.setAuthor(knowledge.getAuthor());
        knowledgeVo.setCreateTime(knowledge.getCreateTime());
        knowledgeVo.setViews(knowledge.getViews());
        // 有文件,截取文件名后缀
        String attachmentName = knowledge.getAttachmentName();
        if (attachmentName != null){
            String[] strArray = attachmentName.split("\\.");
            int suffixIndex = strArray.length - 1;
            //如果是word文件,后缀名修改为word
            if(strArray[suffixIndex].equals("doc") || strArray[suffixIndex].equals("docx")){
                strArray[suffixIndex] = "word";
            }
            knowledgeVo.setAttachmentType(strArray[suffixIndex]);
        }
        //添加分类信息
        List<String> category = new ArrayList<>();
        KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryService.findAllParentByCategoryId(knowledge.getCategoryId());
        String name3 = knowledgeCategoryAdminVo.getName();
        category.add(name3);
        knowledgeVo.setCategory(category);
        if (knowledgeCategoryAdminVo.getParents() != null){
            String name2 = knowledgeCategoryAdminVo.getParents().getName();
            category.add(0,name2);
            knowledgeVo.setCategory(category);
            if (knowledgeCategoryAdminVo.getParents().getParents() != null){
                String name1 = knowledgeCategoryAdminVo.getParents().getParents().getName();
                category.add(0,name1);
                knowledgeVo.setCategory(category);
            }
        }
        return knowledgeVo;
    }
}
